@@ -3,8 +3,12 @@ package Sub::Defer;
 use strictures 1;
 use base qw(Exporter);
 use Moo::_Utils;
+use Scalar::Util qw(weaken);
 
-our @EXPORT = qw(defer_sub undefer_sub);
+our $VERSION = '1.004_003';
+$VERSION = eval $VERSION;
+
+our @EXPORT = qw(defer_sub undefer_sub undefer_all);
 
 our %DEFERRED;
 
@@ -13,6 +17,8 @@ sub undefer_sub {
   my ($target, $maker, $undeferred_ref) = @{
     $DEFERRED{$deferred}||return $deferred
   };
+  return ${$undeferred_ref}
+    if ${$undeferred_ref};
   ${$undeferred_ref} = my $made = $maker->();
 
   # make sure the method slot has not changed since deferral time
@@ -23,30 +29,49 @@ sub undefer_sub {
     # _install_coderef calls are not necessary --ribasushi
     *{_getglob($target)} = $made;
   }
-  push @{$DEFERRED{$made} = $DEFERRED{$deferred}}, $made;
+  $DEFERRED{$made} = $DEFERRED{$deferred};
+  weaken $DEFERRED{$made}
+    unless $target;
 
   return $made;
 }
 
+sub undefer_all {
+  undefer_sub($_) for keys %DEFERRED;
+  return;
+}
+
 sub defer_info {
   my ($deferred) = @_;
-  $DEFERRED{$deferred||''};
+  my $info = $DEFERRED{$deferred||''} or return undef;
+  [ @$info ];
 }
 
 sub defer_sub {
   my ($target, $maker) = @_;
   my $undeferred;
-  my $deferred_string;
+  my $deferred_info;
   my $deferred = sub {
-    goto &{$undeferred ||= undefer_sub($deferred_string)};
+    $undeferred ||= undefer_sub($deferred_info->[3]);
+    goto &$undeferred;
   };
-  $deferred_string = "$deferred";
-  $DEFERRED{$deferred} = [ $target, $maker, \$undeferred ];
-  _install_coderef $target => $deferred if defined $target;
+  $deferred_info = [ $target, $maker, \$undeferred, $deferred ];
+  weaken($deferred_info->[3]);
+  weaken($DEFERRED{$deferred} = $deferred_info);
+  _install_coderef($target => $deferred) if defined $target;
   return $deferred;
 }
 
+sub CLONE {
+  %DEFERRED = map { defined $_ && $_->[3] ? ($_->[3] => $_) : () } values %DEFERRED;
+  for my $info (values %DEFERRED) {
+    weaken($info)
+      unless $info->[0] && ${$info->[2]};
+  }
+}
+
 1;
+__END__
 
 =head1 NAME
 
@@ -90,3 +115,27 @@ If the passed coderef has been L<deferred|/defer_sub> this will "undefer" it.
 If the passed coderef has not been deferred, this will just return it.
 
 If this is confusing, take a look at the example in the L</SYNOPSIS>.
+
+=head2 undefer_all
+
+ undefer_all();
+
+This will undefer all defered subs in one go.  This can be very useful in a
+forking environment where child processes would each have to undefer the same
+subs.  By calling this just before you start forking children you can undefer
+all currently deferred subs in the parent so that the children do not have to
+do it.
+
+=head1 SUPPORT
+
+See L<Moo> for support and contact information.
+
+=head1 AUTHORS
+
+See L<Moo> for authors.
+
+=head1 COPYRIGHT AND LICENSE
+
+See L<Moo> for the copyright and license.
+
+=cut
